# dicemusic
### Mark Probert <mark at archsys dot net>

Welcome to the wonderful world of 18th century parlor games. During the later half of that century, for a certain class of people within certain parts of Europe, there was a craze for mixing music with dice thus creating a marvelous diversion for all present.  Rather than playing known piece from Herr Mozart or Mr Handel, the gathered crowd could get their book of DIY music, roll some dice, and then hear the results. 

The goal of this program is to simulate one such work: a book of Mozart Waltzes (Musikalisches Würfelspiel, K.516f). 

...ha! and it's all there, better, at opus-infinity.org

Anyhow, look at that site if you just want the music, else here is some description of this code.

## What It Does

_dicemusic_ is a simple generator of a Lilypond score. You load it into your Common Lisp environment and run one of the generating functions and it spits out, either to a file or to the terminal, a stream of Lilypond fragments you then compile to create the score, plus a MIDI file if requested.

By default the output goes into the directory _output_ within the _dicemusic_ folder.

## Prerequisites

This code has been tested against:

 * SBCL 1.5.5
 * quicklisp "2015-01-08"
 * Lilypond 2.19.83
 
Given the code is fairly simple standard common lisp (it pulls in no dependencies) and the Lilypond syntax I use is also pretty simplistic, you should have few problems using other lisps or versions of Lilypond.  That said, I offer neither guarantees nor help.

While not required, _quicklisp_ makes life easier...

## Using _dicemusic_

Here is a sample session:

`
$ sbcl
This is SBCL 1.5.5, an implementation of ANSI Common Lisp.

* (ql:quickload :dicemusic)
To load "dicemusic":
  Load 1 ASDF system:
    dicemusic
; Loading "dicemusic"
..
(:DICEMUSIC)

* (in-package :dicemusic)
#<PACKAGE "DICEMUSIC">

* (create-score)
Printed score to file: output/WaltzermaticCF46-8BED7368.ly
NIL

* (in-package :cl-user)
#<PACKAGE "COMMON-LISP-USER">
* (quit)

$ cd output

$ ls
WaltzermaticCF46-8BED7368.ly

$ lilypond -s WaltzermaticCF46-8BED7368.ly

$ ls
WaltzermaticCF46-8BED7368.ly	WaltzermaticCF46-8BED7368.pdf
WaltzermaticCF46-8BED7368.midi
`
Easy :-)

## Program Structure

This is a simple program that randomly chooses fragments of music (stored in a hashtable (*mwf*) indexed by a sequence number) based on indexes in a lookup table (*mozart-waltz*). 

To create a piece we generate a sequence of random fragment indexes (_piece-seq_) which we then feed to the Lilypond generation functions (mostly _gen-score_) which then handles all the score structure details and feeding the fragments.

The Lilypond score is structured in a fairly standard manner for piano music using the _ParallelMusic_ concept. If MIDI is requested then _unfoldRepeats_ is used to create the repeats.

## Function Reference

The following functions are exported from the _dicemusic_ package:

### create-score

This is a convenience function that creates a score file, and a MIDI file using reasonable defaults.

### dump-fragments

`(defun dump-fragments (&key (fname "fragments.ly"))`

Dump the musical fragments to a file _fname_ (defaults shown). No MIDI is produced (really? you'd want that?)

### piece-seq

`(defun piece-seq (&key (table *mozart-waltz*))`

Create a sequence of indexes based on the named table. The structure returned is a list of two lists, one each for the first and second parts of the piece. This follows the Simrock method of two groups of eight bars.

### gen-score

`(defun gen-score (seq &key (outs t) (midi nil)
			(title *piece-title*) (subtitle nil)))`
			
This is the core Lilypond output function. It takes a sequence and then creates a score to the output stream _outs_, optionally producing a version with a MIDI block.

_title_ is the score title; _subtitle_ is the piece title, which defaults to printing the sequence numbers.

### gen-score-to-file

`(defun gen-score-to-file (seq fname &key (midi nil)
				      (title *piece-title*) (subtitle nil)))`
 
This function wraps _gen-score_ with a _with-open-file_ on _fname_ to send the output to the indicated file.

## To Do

Some of the more obvious areas to modify are:

* Add first- and second-time bars
* Allow for more than eight bars per section
* Experiment with different sequence tables

## License

Copyright (c) 2019, archsys pty limited
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the archsys pty limited nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL ARCHSYS PTY LIMITED BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
